<?php
/**
 * The model file of farm module of ChanzhiEPS.
 *
 * @copyright   Copyright 2009-2010 QingDao Nature Easy Soft Network Technology Co,LTD (www.cnezsoft.com)
 * @license     ZPL (http://zpl.pub/page/zplv11.html)
 * @author      Xiying Guan <guanxiying@xirangit.com>
 * @package     farm
 * @version     $Id$
 * @link        http://www.chanzhi.org
 */
class farmModel extends model
{
    /**
     * Check farm by url and private;
     * 
     * @access public
     * @return void
     */
    public function checkFarm()
    {
        $result = true;
        if(!$this->post->private or !$this->post->from) $result = false;
        $farm = $this->getByUrl(helper::safe64Decode($this->post->from));
        if(empty($farm)) $result = false;
        if($farm->private != $this->post->private) $result = false;
        if(!$result) die('Farm private error.');
        return true;
    }

    /**
     * Get farm by ID.
     * 
     * @param  int    $id 
     * @access public
     * @return array
     */
    public function getByID($farmID)
    {
        return $this->dao->select('*')->from(TABLE_FARM)->where('id')->eq($farmID)->fetch();
    }

    /**
     * Get farm by url.
     * 
     * @param  string    $url 
     * @access public
     * @return array
     */
    public function getByUrl($url)
    {
        return $this->dao->select('*')->from(TABLE_FARM)->where('url')->eq($url)->fetch();
    }

    /**
     * Get farm pairs.
     * 
     * @access public
     * @return array
     */
    public function getPairs()
    {
        return $this->dao->select('id, name')->from(TABLE_FARM)->fetchPairs();
    }

    /**
     * Get farm list.
     * 
     * @param  object    $pager 
     * @access public
     * @return array
     */
    public function getList($pager = null)
    {
        return $this->dao->select('*')->from(TABLE_FARM)->page($pager)->fetchAll('id');
    }

    /**
     * Create a farm site.
     * 
     * @access public
     * @return void
     */
    public function create()
    {
        $params = new stdclass();
        $params->m        = 'farm';
        $params->f        = 'register';
        $params->account  = $this->post->account;
        $params->password = md5($this->post->password);
        $params->url      = helper::safe64Encode(commonModel::getSysUrl() . $this->server->script_name);
        $params->name     = $this->config->site->name;

        $registerUrl = $this->post->url . '?'. http_build_query($params);
        if(substr($registerUrl, 0, 4) != 'http') $registerUrl = 'http://' . $registerUrl;
        $result = file_get_contents($registerUrl);
        $result = json_decode($result);

        if(!is_object($result)) return array('result' => 'fail', 'message' => sprintf(zget($this->lang->farm->errors, 'register'), parse_url($registerUrl,  PHP_URL_HOST)));
        if($result->result !== 'success') return array('result' => 'fail', 'message' => zget($this->lang->farm->errors, $result->error));

        $farm = fixer::input('post')->add('private', $result->private)->get();
        if(substr($farm->url, 0, 4) != 'http') $registerUrl = 'http://' . $registerUrl;

        $this->dao->replace(TABLE_FARM)->data($farm, 'account,password')->batchCheck($this->config->farm->require->create, 'notempty')->exec();
        if(dao::isError()) return array('result' => 'fail', 'message' => dao::getError());
        return array('result' => 'success', 'message' => $this->lang->saveSuccess, 'locate' => inlink('admin'));
    }

    /**
     * Update a farm.
     * 
     * @param  int    $farmID 
     * @access public
     * @return void
     */
    public function update($farmID)
    {
        $farm = fixer::input('post')->get();
        $this->dao->update(TABLE_FARM)->data($farm)->where('id')->eq($farmID)->exec();
        return !dao::isError();
    }

    /**
     * Get tree of a farm
     * 
     * @param  int    $farmID 
     * @param  string    $type 
     * @access public
     * @return void
     */
    public function getTreeFromFarm($farmID, $type)
    {
        $params = new stdclass();
        $params->type = $type;
        $result = $this->post($farmID, 'apigetTree', $params);
        if(!is_object($result)) return false;
        if($result->result == 'success') return $result->data;
        return false;
    }

    /**
     * Sync an object to a farm site.
     * 
     * @param  int       $farmID 
     * @param  object    $object 
     * @access public
     * @return object
     */
    public function syncObject($farmID, $object)
    {
        $result = $this->post($farmID, 'apiSyncObject', $object);
        if(!is_object($result)) return array('result' => 'fail', 'message' => $this->lang->farm->errors->sync);
        return $result;
    }

    /**
     * Save object post from farm.
     * 
     * @access public
     * @return object
     */
    public function saveObject()
    {
        $syncType = $this->post->syncType;
        if(is_callable(array($this, "save{$syncType}Object"))) return call_user_func(array($this, "save{$syncType}Object"));
    }

    /**
     * Save article object from farm.
     * 
     * @param  string $type 
     * @access public
     * @return object
     */
    public function saveArticleObject($type = 'article')
    {
        $article = fixer::input('post')
            ->join('categories', ',')
            ->add('type', $type)
            ->stripTags('content,link,videoLink', $this->config->allowedTags->admin)
            ->removeIF($type == 'video', 'videoLink, width, height, autoplay')
            ->remove('syncType,from,private')
            ->get();

        $article->addedDate  = helper::now();
        $article->editedDate = $article->addedDate;

        /* Set video */
        if($type == 'video')
        {
            if(!$this->post->videoLink)
            {
                dao::$errors['videoLink'] = sprintf($this->lang->error->notempty, $this->lang->video->link);
                return false;
            }
    
            $video = new stdclass();
            $video->link     = $this->post->videoLink;
            $video->width    = $this->post->width;
            $video->height   = $this->post->height;
            $video->autoplay = $this->post->autoplay ? true : false;
            $article->video = helper::jsonEncode($video);
        }

        $article->keywords = seo::unify($article->keywords, ',');
        if(!empty($article->alias)) $article->alias = seo::unify($article->alias, '-');
        $article->content = $this->loadModel('article')->rtrimContent($article->content);
    
        $this->dao->insert(TABLE_ARTICLE)
            ->data($article, $skip = 'categories,uid,isLink')
            ->autoCheck()
            ->batchCheckIF($type != 'page' and $type != 'submission' and !$this->post->isLink, $this->config->article->require->create, 'notempty')
            ->batchCheckIF($type == 'submission', $this->config->article->require->post, 'notempty')
            ->batchCheckIF($type == 'page' and !$this->post->isLink, $this->config->article->require->page, 'notempty')
            ->batchCheckIF($type != 'page' and $this->post->isLink, $this->config->article->require->link, 'notempty')
            ->batchCheckIF($type == 'page' and $this->post->isLink, $this->config->article->require->pageLink, 'notempty')
            ->checkIF(($type == 'page') and $this->post->alias, 'alias', 'unique', "type='page'")
            ->exec();

        $articleID = $this->dao->lastInsertID();
        if($type != 'page') $this->article->processCategories($articleID, $type, $this->post->categories);

        $article = $this->article->getByID($articleID);
        $this->loadModel('search')->save($type, $article);
        if(!dao::isError()) return array('result' => 'success', 'copyURL' => rtrim(commonModel::getSysUrl(), '/') . $this->article->createPreviewLink($articleID), 'copySite' => $this->config->site->name);
        return array('result' => 'fail', 'message' => dao::getError());
    }

    /**
     * Save page object.
     * 
     * @access public
     * @return object
     */
    public function savePageObject()
    {
        return $this->saveArticleObject('page');
    }

    /**
     * Save blog object.
     * 
     * @access public
     * @return object
     */
    public function saveBlogObject()
    {
        return $this->saveArticleObject('blog');
    }

    /**
     * Post a farm request.
     * 
     * @param  int       $farmID 
     * @param  string    $method 
     * @param  object    $params 
     * @access public
     * @return object
     */
    public function post($farmID, $method, $params)
    {
        $farm = $this->getByID($farmID);       
        if(empty($farm)) return false;
        if(substr($farm->url, 0, 4) != 'http') $farm->url = 'http://' . $farm->url;
        $params->private = $farm->private;
        $params->from    = helper::safe64Encode(commonModel::getSysUrl() . $this->server->script_name);
        $api = $farm->url . "?m=farm&f=$method";

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $api);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query((array)$params));

        $response = curl_exec($curl);
        curl_close($curl);
        
        return json_decode($response);
    }
}
